# Generated by Selenium IDE
from selenium.webdriver.common.by import By

from splent_framework.environment.host import get_host_for_selenium_testing
from splent_framework.selenium.common import initialize_driver


class TestSignup:
    def setup_method(self, method):
        self.driver = initialize_driver()
        self.vars = {}

    def teardown_method(self, method):
        self.driver.quit()

    def test_signup(self):
        self.driver.get(get_host_for_selenium_testing())
        self.driver.find_element(By.LINK_TEXT, "Sign Up").click()
        self.driver.find_element(By.ID, "email").send_keys("user1@example.com")
        self.driver.find_element(By.ID, "password").send_keys("1234")
        self.driver.find_element(By.ID, "name").click()
        self.driver.find_element(By.ID, "name").send_keys("Hola")
        self.driver.find_element(By.ID, "surname").send_keys("Juan")
        self.driver.find_element(By.ID, "email").send_keys(
            "holajuan@example.com"
        )
        self.driver.find_element(By.ID, "password").send_keys("holajuan")
        self.driver.find_element(By.ID, "submit").click()



import os

from flask_login import login_user
from flask_login import current_user

from splent_io.splent_feature_auth.models import User
from splent_io.splent_feature_auth.repositories import UserRepository
from splent_io.splent_feature_profile.models import UserProfile
from splent_io.splent_feature_profile.repositories import UserProfileRepository
from splent_framework.configuration.configuration import uploads_folder_name
from splent_framework.services.BaseService import BaseService


class AuthenticationService(BaseService):

    def __init__(self, user_repository=None, user_profile_repository=None):
        super().__init__(user_repository or UserRepository())
        self.user_profile_repository = user_profile_repository or UserProfileRepository()


    def login(self, email, password, remember=True):
        user = self.repository.get_by_email(email)
        if user is not None and user.check_password(password):
            login_user(user, remember=remember)
            return True
        return False

    def is_email_available(self, email: str) -> bool:
        return self.repository.get_by_email(email) is None

    def create_with_profile(self, **kwargs):
        try:
            email = kwargs.pop("email", None)
            password = kwargs.pop("password", None)
            name = kwargs.pop("name", None)
            surname = kwargs.pop("surname", None)

            if not email:
                raise ValueError("Email is required.")
            if not password:
                raise ValueError("Password is required.")
            if not name:
                raise ValueError("Name is required.")
            if not surname:
                raise ValueError("Surname is required.")

            user_data = {
                "email": email,
                "password": password,
                "active": False,
            }

            profile_data = {
                "name": name,
                "surname": surname,
            }

            user = self.create(commit=False, **user_data)
            profile_data["user_id"] = user.id
            self.user_profile_repository.create(**profile_data)
            self.repository.session.commit()
        except Exception as exc:
            self.repository.session.rollback()
            raise exc
        return user

    def update_profile(self, user_profile_id, form):
        if form.validate():
            updated_instance = self.update(user_profile_id, **form.data)
            return updated_instance, None

        return None, form.errors

    def get_authenticated_user(self) -> User | None:
        if current_user.is_authenticated:
            return current_user
        return None

    def get_authenticated_user_profile(self) -> UserProfile | None:
        if current_user.is_authenticated:
            return current_user.profile
        return None

    def temp_folder_by_user(self, user: User) -> str:
        return os.path.join(uploads_folder_name(), "temp", str(user.id))

    def get_by_email(self, email: str, active: bool = True) -> User:
        return self.repository.get_by_email(email, active)



from flask import flash, render_template, redirect, url_for, request
from flask_login import current_user, logout_user
from pymysql import IntegrityError

from splent_io.splent_feature_auth import auth_bp
from splent_io.splent_feature_auth.decorators import guest_required
from splent_io.splent_feature_auth.forms import SignupForm, LoginForm
from splent_io.splent_feature_auth.services import AuthenticationService
from splent_io.splent_feature_confirmemail.services import ConfirmemailService
from splent_framework.db import db


authentication_service = AuthenticationService()
confirmemail_service = ConfirmemailService()


@auth_bp.route("/signup/", methods=["GET", "POST"])
def show_signup_form():
    if current_user.is_authenticated:
        return redirect(url_for("public.index"))

    form = SignupForm()
    if form.validate_on_submit():

        email = form.email.data
        if not authentication_service.is_email_available(email):
            flash(f"Email {email} is already in use", "danger")
            return render_template("auth/signup_form.html", form=form)

        try:
            # We try to create the user
            user = authentication_service.create_with_profile(**form.data)
            confirmemail_service.send_confirmation_email(user.email)
        except IntegrityError as exc:
            db.session.rollback()
            if "Duplicate entry" in str(exc):
                flash(f"Email {email} is already in use", "danger")
            else:
                flash(f"Error creating user: {exc}", "danger")
            return render_template("auth/signup_form.html", form=form)

        return redirect(url_for("public.index"))

    return render_template("auth/signup_form.html", form=form)


@auth_bp.route("/login", methods=["GET", "POST"])
@guest_required
def login():
    if current_user.is_authenticated:
        return redirect(url_for("public.index"))

    form = LoginForm()
    if request.method == "POST" and form.validate_on_submit():

        if authentication_service.login(form.email.data, form.password.data):
            return redirect(url_for("public.index"))

        return render_template(
            "auth/login_form.html", form=form, error="Invalid credentials"
        )

    return render_template("auth/login_form.html", form=form)


@auth_bp.route("/logout")
def logout():
    logout_user()
    return redirect(url_for("public.index"))



from splent_framework.blueprints.base_blueprint import BaseBlueprint
from flask_login import LoginManager

auth_bp = BaseBlueprint("auth", __name__, template_folder="templates")


def init_feature(app):
    from .models import User
    login_manager = LoginManager()
    login_manager.init_app(app)
    login_manager.login_view = "auth.login"

    @login_manager.user_loader
    def load_user(user_id):
        return User.query.get(int(user_id))
    
    app.login_manager = login_manager
    
def inject_context_vars(app):
    return {}


def test_login_sets_session_cookie(test_client, auth_test_user):
    response = test_client.post("/login", data={
        "email": "user1@example.com",
        "password": "1234"
    }, follow_redirects=True)

    assert b"Edit profile" in response.data


from datetime import datetime

from flask_login import UserMixin
import pytz
from splent_io.splent_feature_profile.models import UserProfile
from splent_framework.db import db
from werkzeug.security import generate_password_hash, check_password_hash



class User(db.Model, UserMixin):
    id = db.Column(db.Integer, primary_key=True)

    email = db.Column(db.String(256), unique=True, nullable=True)
    password = db.Column(db.String(256), nullable=False)
    created_at = db.Column(
        db.DateTime, nullable=False, default=lambda: datetime.now(pytz.utc)
    )

    profile = db.relationship(UserProfile, backref="user", uselist=False)
    active = db.Column(db.Boolean, default=True, nullable=False)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if "password" in kwargs:
            self.set_password(kwargs["password"])

    def __repr__(self):
        return f"<User {self.email}>"

    def set_password(self, password):
        self.password = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password, password)

    def temp_folder(self) -> str:
        import os
        from splent_framework.configuration.configuration import uploads_folder_name
        return os.path.join(uploads_folder_name(), "temp", str(self.id))



from splent_framework.fixtures.fixtures import *
import pytest
from splent_io.splent_feature_auth.models import User
from splent_framework.db import db


@pytest.fixture(scope="function")
def auth_test_user(test_app):
    with test_app.app_context():
        user = User(email="user1@example.com", active=True)
        user.set_password("1234")
        db.session.add(user)
        db.session.commit()
        return user

@pytest.fixture(scope="function")
def logged_in_client(test_client, auth_test_user):
    test_client.post("/login", data={
        "email": "user1@example.com",
        "password": "1234"
    }, follow_redirects=True)
    return test_client

@pytest.fixture(scope="function")
def inactive_user(test_app):
    with test_app.app_context():
        user = User(email="inactive@example.com", active=False)
        user.set_password("1234")
        db.session.add(user)
        db.session.commit()
        return user
